/*
 *  Copyright (C) 2005 DFKI GmbH
 *  Image Understanding and Pattern Recognition Lab
 *  Author: Christian Kofler <christian.kofler@dfki.de>
 *  wxUnicode Support: Thomas Deselaers <deselaers@informatik.rwth-aachen.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <wx/wx.h>

#include "Application.h"

using namespace std;


BEGIN_EVENT_TABLE(RootFrame, wxFrame)
  EVT_PAINT(RootFrame::OnPaint)
  EVT_BUTTON(101, RootFrame::OnRandom)
  EVT_BUTTON(wxID_ANY, RootFrame::OnClick)
  EVT_COMMAND_SCROLL(102, RootFrame::OnZoom)
  END_EVENT_TABLE()


  IMPLEMENT_APP(wxApplication)

  char host[128];
unsigned int port;

bool wxApplication::OnInit() {

  if (argc < 2) {
    cerr << "usage: firewatch host [port]" << endl << endl;
    exit(-1);
  } else {
    port = 12960;
    if (argc == 3) {
      sscanf(wxString(argv[2]).mb_str(), "%d", &port);
    }
    wxString tmp(argv[1]);
    strcpy(host, tmp.mb_str());
  }


  //--- set up GUI
  RootFrame* frame = new RootFrame(wxT("FireWatch"), 150, 150, 670, 560);
  frame->Centre();
  frame->Show(true);
  SetTopWindow(frame);

    
  frame->Refresh();
  return true;
}

wxApplication::~wxApplication() {

}

RootFrame::RootFrame(const wxChar *title, int xpos, int ypos, int width, 
                     int height) : wxFrame( (wxFrame *) NULL, -1,  title,
                                            wxPoint(xpos, ypos), wxSize(width, height) ) {
  // add handler to operate on jpegs
  wxImage::AddHandler(new wxJPEGHandler);
  wxImage::AddHandler(new wxPNGHandler);

  firewatch.setServer(host, port);
  firewatch.initialize();
    
  imageScale = 0.4f;
    
  topsizer = new wxBoxSizer( wxVERTICAL );

  wxBoxSizer *control_sizer = new wxBoxSizer( wxHORIZONTAL );
  control_sizer->Add(
                     new wxButton( this, 101, wxT("Random Images") ),
                     0,           // make horizontally unstretchable
                     wxALL,       // make border all around (implicit top alignment)
                     10 );        // set border width to 10
        
  control_sizer->Add(    
                     new wxSlider(this, 102, 40, 30, 50, wxDefaultPosition, wxSize(150,30), wxSL_HORIZONTAL, wxDefaultValidator, wxT("slider")),
                     0, wxALL, 10);
        
  topsizer->Add(
                control_sizer,
                0,                // make vertically unstretchable
                wxALIGN_CENTER ); // no border and centre horizontally

  imageSizer = new wxFlexGridSizer(5);

  topsizer->Add(
                imageSizer,
                1,                // make vertically stretchable
                wxALIGN_CENTER ); // no border and centre horizontally

  SetSizer( topsizer );      // use the sizer for layout

  topsizer->SetSizeHints( this );   // set size hints to honour minimum size
}

RootFrame::~RootFrame() {
  firewatch.release();
}

void RootFrame::OnPaint(wxPaintEvent& event) {
  // get device context
  wxPaintDC dc(this);

}

void RootFrame::OnRandom(wxCommandEvent& event) {
  vector<ResultImage>& results = firewatch.getRandomImages();
  string path = firewatch.getPath();
    
  imageButtons.clear();
    
  for (unsigned int i = 0; i < results.size(); ++i) {
    string filename = path + "/" + results[i].getName();
    wxString tmpwxs(filename.c_str(),wxConvUTF8);
    wxImage* tmpWxImage = new wxImage( tmpwxs );
    tmpWxImage->Rescale(tmpWxImage->GetWidth() * imageScale, tmpWxImage->GetHeight() * imageScale);
    tmpwxs=wxString(results[i].getName().c_str(),wxConvUTF8);
    wxBitmapButton* tmpBmp = new wxBitmapButton(this, wxID_HIGHEST +1+i, wxBitmap(tmpWxImage), wxDefaultPosition, 
                                                wxDefaultSize, wxBU_BOTTOM, wxDefaultValidator, tmpwxs);
    imageButtons.push_back(tmpBmp);
    imageSizer->Show(i, false);
    imageSizer->Remove(i);
    imageSizer->Insert(i, tmpBmp, 0, wxALIGN_CENTER);
        
    delete tmpWxImage;
  }
  topsizer->SetSizeHints( this );   // set size hints to honour minimum size
  Centre();
}

void RootFrame::OnClick(wxCommandEvent& event) {
  if(event.GetId() < wxID_HIGHEST) return;
    
  string file (wxString(((wxWindow*)event.GetEventObject())->GetName()).mb_str());
    
  cout << file << endl;

  vector<ResultImage>& results = firewatch.retrieve(file);
  string path = firewatch.getPath();
    
  imageButtons.clear();
    
  for (unsigned int i = 0; i < results.size(); ++i) {
    string filename = path + "/" + results[i].getName();
    wxString tmpwxs(filename.c_str(),wxConvUTF8);
    wxImage* tmpWxImage = new wxImage(tmpwxs);
    tmpWxImage->Rescale(tmpWxImage->GetWidth() * imageScale, tmpWxImage->GetHeight() * imageScale);

    tmpwxs=wxString(results[i].getName().c_str(),wxConvUTF8);
    wxBitmapButton* tmpBmp = new wxBitmapButton(this, wxID_HIGHEST +1+i, wxBitmap(tmpWxImage), wxDefaultPosition, 
                                                wxDefaultSize, wxBU_BOTTOM, wxDefaultValidator, tmpwxs);
    char scoreText[15];
    sprintf(scoreText, "%f", results[i].getScore());

    
    tmpwxs=wxString(scoreText,wxConvUTF8);
    tmpBmp->SetToolTip(tmpwxs);
    imageButtons.push_back(tmpBmp);
    imageSizer->Show(i, false);
    imageSizer->Remove(i);
    imageSizer->Insert(i, tmpBmp, 0, wxALIGN_CENTER);
        
    delete tmpWxImage;
  }
  topsizer->SetSizeHints( this );   // set size hints to honour minimum size
  Centre();
    
}

void RootFrame::OnZoom(wxScrollEvent& event) {
  imageScale = (float)event.GetPosition() / 100.f;

}
