/*
 *  Copyright (C) 2005 DFKI GmbH
 *  Image Understanding and Pattern Recognition Lab
 *  Author: Christian Kofler <christian.kofler@dfki.de>
 *  wxUnicode Support: Thomas Deselaers <deselaers@informatik.rwth-aachen.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <netdb.h>

#include <iostream>

using namespace std;

#include "FireClient.h"

FireClient::FireClient() {
    // init some general server struct fields
    server.sin_family = AF_INET;
    // set zero fields in server struct
    memset(&(server.sin_zero), '\0', 8);
    
    rcvBuffer = 0;
    bufferSize = 1024;
    connected = false;
}

FireClient::~FireClient() {
    if (rcvBuffer != 0) delete[] rcvBuffer;
}

bool FireClient::connect(char* host, unsigned short port) {
    struct hostent *he;

    if (host == 0 || port == 0) return false;

    he = gethostbyname(host);

    if (he == 0) {
        cerr << "[Networker] Could not get IP of host " << host;
        cerr << " (error: " << strerror(errno) << ")" << endl;
        return false;
    }

    socketID = socket(AF_INET, SOCK_STREAM, 0);

    if (socketID == -1) {
        cerr << "[Networker] Could not create socket";
        cerr << " (error: " << strerror(errno) << ")" << endl;
    }

    server.sin_port = htons(port);
    server.sin_addr = *((struct in_addr *)he->h_addr);

    if (::connect(socketID, (struct sockaddr *)&server, sizeof(struct sockaddr)) == -1) {
        cerr << "[Networker] Could not connect to " << host << " (" << port << ")" << endl;
    } else {
        connected = true;
    }
    
    return connected;
}

char* FireClient::request(string query) {
    if (query.empty()) return 0;
    
    int byteCount = 0;

    query.append("\r\n");
    
    // assume that entire query can be send at once! Note that therefore
    // the query must be less than 1 kB in size!!!
    byteCount = send(socketID, (char*)query.c_str(), query.length(), 0);
    if (byteCount == -1) {
        cerr << "[Networker] Could not send data";
        cerr << " (error: " << strerror(errno) << ")" << endl;
        return 0;
    }
    
    byteCount = 0;

    if (rcvBuffer != 0) delete[] rcvBuffer;
    rcvBuffer = new char[bufferSize];

    byteCount = recv(socketID, rcvBuffer, bufferSize-1, 0);
    if (byteCount == -1) {
        cerr << "[Networker] Could not receive data";
        cerr << " (error: " << strerror(errno) << ")" << endl;
        return 0;
    }
    rcvBuffer[byteCount] = '\0';
    
    return rcvBuffer;
}

bool FireClient::release() {
    
    // be kind and say goodbye to the fire server
    send(socketID, "bye\r\n", 5, 0);

    if (close(socketID) == -1) {
        cerr << "[Networker] Could not close connection";
        cerr << " (error: " << strerror(errno) << ")" << endl;
        return false;
    }
    
    return true;
}
