/*
 *  Copyright (C) 2005 DFKI GmbH
 *  Image Understanding and Pattern Recognition Lab
 *  Author: Christian Kofler <christian.kofler@dfki.de>
 *  wxUnicode Support: Thomas Deselaers <deselaers@informatik.rwth-aachen.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "FireWatch.h"

#include <cstdlib>
#include <iostream>
#include <iomanip>

using namespace std;

FireWatch::FireWatch() {
    hostname = "toronto";
    defaultPort = 12960;
    
    fileCount = 0;
    resultCount = 0;
    
    resultImages.clear();
    
    setlocale(LC_NUMERIC, "C");
}

FireWatch::~FireWatch() {
    if (resultImages.empty()) resultImages.clear();
}

void FireWatch::setServer(char* host, unsigned short port) {
    if(host == 0) return;
    
    hostname = host;
    defaultPort = port;
}

bool FireWatch::initialize() {
    bool status = false;
    
    status = client.connect((char*)hostname.c_str(), defaultPort);
    
    getServerConfig();
    if (path.empty()) return false;
    
    cout << "fileCount: " << fileCount << endl;
    cout << "resultCount: " << resultCount << endl;
    cout << "path: " << path << endl;

    return status;
}

bool FireWatch::release() {
    bool status = false;
    
    status = client.release();
    if (status) cout << "connection closed" << endl;
    else cout << "failed to close connection!" << endl;
    
    return status;
}

vector<ResultImage>& FireWatch::getRandomImages() {
    char* result;
    string token;
    
    result = client.request("random");
    
    token = strtok(result, " ");
    if (!token.empty()) {
        if (!resultImages.empty()) resultImages.clear();
        resultImages.push_back(ResultImage(token, 0.f));
        
        for (int i = 1; i < resultCount; ++i) {
            token = strtok(0, " ");
            if (token.empty()) break;
            resultImages.push_back(ResultImage(token, 0.f));
        }
    }

    printResultImages();
    
    return resultImages;
}

vector<ResultImage>& FireWatch::retrieve(string queryImage) {
    char* result;
    char* token;
    string name;
    double score = 0.f;
    
    string query = "retrieve +" + queryImage;
    
    result = client.request(query);
    
    cout << result << endl;
    
    token = strtok(result, " ");
    if (token) {
        if (!resultImages.empty()) resultImages.clear();
        name = token;
        token = strtok(0, " ");
//        sscanf(token, "%f", &score);
        score = atof(token);
        resultImages.push_back(ResultImage(name, score));
        
        for (int i = 1; i < resultCount; ++i) {
            name = strtok(0, " ");
            if (name.empty()) break;
            token = strtok(0, " ");
            score = atof(token);

            ResultImage resImg(name, score);
            resImg.setScore(score);
            resultImages.push_back(resImg);
        }
    }

    printResultImages();
    
    return resultImages;
}

void FireWatch::getServerConfig() {
    char* result;
    char* token;

    result = client.request("info");
    
    cout << result << endl;
    
    // cut "filelist"
    strtok(result, " ");
    token = strtok(0, " ");
    sscanf(token, "%d", &fileCount);
    // cut "results"
    strtok(0, " ");
    token = strtok(0, " ");
    sscanf(token, "%d", &resultCount);
    // cut "path"
    strtok(0, " ");
    path = strtok(0, " ");
}

void FireWatch::printResultImages() {
    cout << endl << "result images: " << endl;
    for (unsigned int i = 0; i < resultImages.size(); ++i) {
        cout << "name: " << resultImages[i].getName();
        cout << " score: " << resultImages[i].getScore() << endl;
    }    
}

